@@ -68,6 +68,7 @@ char *acpi_labels_20020214[] = {
 	"thermal_zone",
 #endif
 	"state:",
+	"last full capacity:",
 	NULL
 };
 
@@ -88,7 +89,7 @@ inline char *get_acpi_file (const char *file) {
 	fd = open(file, O_RDONLY);
 	if (fd == -1) return NULL;
 	end = read(fd, buf, sizeof(buf));
-	buf[end] = '\0';
+	buf[end-1] = '\0';
 	close(fd);
 	return buf;
 }
@@ -112,7 +113,7 @@ inline char *scan_acpi_value (const char *buf, const char *key) {
 	static char ret[256];
 	
 	if ((ptr = strstr(buf, key))) {
-		if (sscanf(ptr + strlen(key), "%s", ret) == 1)
+		if (sscanf(ptr + strlen(key), "%255s", ret) == 1)
 			return ret;
 	}
 	return NULL;
@@ -128,18 +129,32 @@ char *get_acpi_value (const char *file, const char *key) {
 	return scan_acpi_value(buf, key);
 }
 
-/* Returns the design capacity of a battery. */
+/* Returns the maximum capacity of a battery.
+ *
+ * Note that this returns the highest possible capacity for the battery,
+ * even if it can no longer charge that fully. So normally it uses the
+ * design capacity. While the last full capacity of the battery should
+ * never exceed the design capacity, some silly hardware might report
+ * that it does. So if the last full capacity is greater, it will be
+ * returned.
+ */
 int get_acpi_batt_capacity(int battery) {
-	int cap;
-	char *caps=get_acpi_value(acpi_batt_info[battery], acpi_labels[label_design_capacity]);
-	if (caps == NULL)
-		cap=0; /* battery not present */
+	int dcap, lcap;
+	char *dcaps=get_acpi_value(acpi_batt_info[battery], acpi_labels[label_design_capacity]);
+	char *lcaps=get_acpi_value(acpi_batt_info[battery], acpi_labels[label_last_full_capacity]);
+	if (dcaps == NULL)
+		dcap=0; /* battery not present */
 	else
-		cap=atoi(caps);
+		dcap=atoi(dcaps);
 	/* This is ACPI's broken way of saying that there is no battery. */
-	if (cap == 655350)
+	if (dcap == 655350)
 		return 0;
-	return cap;
+	if (lcaps != NULL) {
+		lcap=atoi(lcaps);
+		if (lcap > dcap)
+			return lcap;
+	}
+	return dcap;
 }
 
 /* Comparison function for qsort. */
@@ -168,8 +183,8 @@ int find_items (char *itemname, char infoarray[ACPI_MAXITEM][128],
 	if (dir == NULL)
 		return 0;
 	while ((ent = readdir(dir))) {
-		if (!strncmp(".", ent->d_name, 1) || 
-		    !strncmp("..", ent->d_name, 2))
+		if (!strcmp(".", ent->d_name) || 
+		    !strcmp("..", ent->d_name))
 			continue;
 
 		devices[num_devices]=strdup(ent->d_name);
@@ -198,7 +213,6 @@ int find_items (char *itemname, char infoarray[ACPI_MAXITEM][128],
 int find_batteries(void) {
 	int i;
 	acpi_batt_count = find_items(acpi_labels[label_battery], acpi_batt_info, acpi_batt_status);
-	/* Read in the last charged capacity of the batteries. */
 	for (i = 0; i < acpi_batt_count; i++)
 		acpi_batt_capacity[i] = get_acpi_batt_capacity(i);
 	return acpi_batt_count;
@@ -245,10 +259,16 @@ int acpi_supported (void) {
 	}
 	closedir(dir);
 	
-	version = get_acpi_value(PROC_ACPI "/info", "ACPI-CA Version:");
+	/* If kernel is 2.6.21 or newer, version is in
+	   /sys/module/acpi/parameters/acpica_version */
+	
+	version = get_acpi_file("/sys/module/acpi/parameters/acpica_version");
 	if (version == NULL) {
-		/* 2.5 kernel acpi */
-		version = get_acpi_value(PROC_ACPI "/info", "version:");
+		version = get_acpi_value(PROC_ACPI "/info", "ACPI-CA Version:");
+		if (version == NULL) {
+			/* 2.5 kernel acpi */
+			version = get_acpi_value(PROC_ACPI "/info", "version:");
+		}
 	}
 	if (version == NULL) {
 		return 0;
@@ -281,6 +301,16 @@ int acpi_supported (void) {
 int acpi_read (int battery, apm_info *info) {
 	char *buf, *state;
 	
+	if (acpi_batt_count == 0) {
+		info->battery_percentage = 0;
+		info->battery_time = 0;
+		info->battery_status = BATTERY_STATUS_ABSENT;
+		acpi_batt_capacity[battery] = 0;
+		/* Where else would the power come from, eh? ;-) */
+		info->ac_line_status = 1;
+		return 0;
+	}
+	
 	/* Internally it's zero indexed. */
 	battery--;
 	
@@ -332,7 +362,10 @@ int acpi_read (int battery, apm_info *info) {
 				info->battery_status = BATTERY_STATUS_CHARGING;
 				info->ac_line_status = 1;
 				info->battery_flags = info->battery_flags | BATTERY_FLAGS_CHARGING;
-				info->battery_time = -1 * (float) (acpi_batt_capacity[battery] - pcap) / (float) rate * 60;
+				if (rate)
+					info->battery_time = -1 * (float) (acpi_batt_capacity[battery] - pcap) / (float) rate * 60;
+				else
+					info->battery_time = 0;
 				if (abs(info->battery_time) < 0.5)
 					info->battery_time = 0;
 			}
@@ -371,8 +404,13 @@ int acpi_read (int battery, apm_info *info) {
 			 * buf below this point! */
 			acpi_batt_capacity[battery] = get_acpi_batt_capacity(battery);
 		}
+		else if (pcap > acpi_batt_capacity[battery]) {
+			/* Battery is somehow charged to greater than max
+			 * capacity. Rescan for a new max capacity. */
+			find_batteries();
+		}
 		
-		if (pcap) {
+		if (pcap && acpi_batt_capacity[battery]) {
 			/* percentage = (current_capacity / max capacity) * 100 */
 			info->battery_percentage = (float) pcap / (float) acpi_batt_capacity[battery] * 100;
 		}
